from django.apps import apps
from django.contrib import admin
from .conf import settings
from .models import BaseTenant
from .utils import get_custom_apps



class _DefaultTenantOnlyAdmin(admin.ModelAdmin):
    """
    Internal admin to hide models outside the default tenant.
    """

    def _is_default_tenant(self, request):
        tenant: BaseTenant = request.tenant
        return tenant.name == settings.PUBLIC_SCHEMA_NAME

    def get_model_perms(self, request):
        if self._is_default_tenant(request):
            return super().get_model_perms(request)
        return {}

    def has_module_permission(self, request):
        return self._is_default_tenant(request)

    def has_view_permission(self, request, obj=None):
        return self._is_default_tenant(request)

    def has_add_permission(self, request):
        return self._is_default_tenant(request)

    def has_change_permission(self, request, obj=None):
        return self._is_default_tenant(request)

    def has_delete_permission(self, request, obj=None):
        return self._is_default_tenant(request)


app_names = get_custom_apps()

for app_name in app_names:
    app_config = apps.get_app_config(app_name)
    if not getattr(app_config, "tenant_managed", True):
        for model in app_config.get_models():
            if admin.site.is_registered(model):
                admin.site.unregister(model)
            admin.site.register(model, _DefaultTenantOnlyAdmin)
    else:
        for model in app_config.get_models():
            if getattr(model, "globally_managed", False):
                continue
            if not getattr(model, "tenant_managed", True):
                if admin.site.is_registered(model):
                    admin.site.unregister(model)
                admin.site.register(model, _DefaultTenantOnlyAdmin)
            


from django.db import connection

from django_omnitenant.models import BaseTenant
from .base import BaseTenantBackend
from django_omnitenant.utils import get_active_schema_name

from django.core.management import call_command
from django_omnitenant.tenant_context import TenantContext


class SchemaTenantBackend(BaseTenantBackend):
    def __init__(self, tenant: BaseTenant):
        super().__init__(tenant)
        self.schema_name = tenant.config.get("schema_name") or tenant.tenant_id

    def bind(self):
        with connection.cursor() as cursor:
            cursor.execute(f'CREATE SCHEMA IF NOT EXISTS "{self.schema_name}"')
        print(f"[SCHEMA BACKEND] Schema '{self.schema_name}' ensured.")

    def create(self, run_migrations=False, **kwargs):
        self.bind()
        super().create(run_migrations=run_migrations, **kwargs)

    def migrate(self, *args, **kwargs):
        with TenantContext.use_tenant(self.tenant):
            call_command("migrate", *args, database="default", **kwargs)
        super().migrate()

    def delete(self, drop_schema=True):
        if drop_schema:
            with connection.cursor() as cursor:
                cursor.execute(f'DROP SCHEMA IF EXISTS "{self.schema_name}" CASCADE')
            print(f"[SCHEMA BACKEND] Schema '{self.schema_name}' dropped.")
            super().delete()

    def activate(self):
        self.bind()
        self.previous_schema = get_active_schema_name(connection)
        connection.set_schema(self.schema_name)
        super().activate()

    def deactivate(self):
        connection.set_schema(self.previous_schema)
        super().deactivate()



from django.core.management.base import BaseCommand
from django_omnitenant.models import BaseTenant
from django_omnitenant.utils import get_tenant_model
from django_omnitenant.utils import get_tenant_backend


class Command(BaseCommand):
    help = "Run migrations for all tenants."

    def handle(self, *args, **options):
        Tenant = get_tenant_model()

        for tenant in Tenant.objects.all():  # type: ignore
            tenant: BaseTenant = tenant
            self.stdout.write(
                self.style.MIGRATE_HEADING(f"Migrating tenant: {tenant.tenant_id}")
            )

            try:
                backend = get_tenant_backend(tenant)
                backend.migrate()
                self.stdout.write(
                    self.style.SUCCESS(
                        f"Tenant '{tenant.tenant_id}' migrated successfully."
                    )
                )
            except Exception as e:
                self.stdout.write(
                    self.style.ERROR(
                        f"Migrations failed for tenant '{tenant.tenant_id}': {e}"
                    )
                )



from django.core.management.commands.shell import Command as ShellCommand

from django_omnitenant.tenant_context import TenantContext
from django_omnitenant.utils import get_tenant_model


class Command(ShellCommand):
    help = "Runs a Django shell with a specific tenant activated."

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument("--tenant-id", type=str, help="Tenant ID to activate")

    def handle(self, *args, **options):
        tenant_id = options.get("tenant_id")
        if tenant_id:
            Tenant = get_tenant_model()
            try:
                tenant = Tenant.objects.get(tenant_id=tenant_id)
            except Tenant.DoesNotExist:
                self.stderr.write(
                    self.style.ERROR(f"Tenant with ID '{tenant_id}' does not exist.")
                )
                return
            with TenantContext.use_tenant(tenant):
                self.stdout.write(
                    self.style.SUCCESS(f"Tenant '{tenant_id}' activated.")
                )
                super().handle(*args, **options)
        else:
            super().handle(*args, **options)



from django.db.backends.postgresql.base import (
    DatabaseWrapper as PostgresDatabaseWrapper,
)
from django_omnitenant.conf import settings


class DatabaseWrapper(PostgresDatabaseWrapper):
    """
    PostgreSQL wrapper that supports switching schemas per tenant
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._current_schema = settings.PUBLIC_SCHEMA_NAME

    def set_schema(self, schema_name):
        """
        Switch the PostgreSQL schema for this connection.
        """
        if not self.is_usable():
            self.ensure_connection()

        with self.cursor() as cursor:
            cursor.execute(f'SET search_path TO "{schema_name}"')

        self._current_schema = schema_name

    def set_schema_to_public(self):
        """
        Reset to the public schema.
        """
        self.set_schema(settings.PUBLIC_SCHEMA_NAME)

    @property
    def current_schema(self):
        return self._current_schema
